/*
    Copyright (C) 2003 Nikolas Zimmermann <wildfox@kde.org>
    This file is part of the KDE project

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    aint with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include <math.h>
#include "Point.h"
#include "BezierPathLibart.h"

#include <kdebug.h>

#include <libart_lgpl/art_vpath.h>
#include <libart_lgpl/art_vpath_bpath.h>

using namespace T2P;

BezierPathLibart::BezierPathLibart() : BezierPath()
{
	end = 0;
	path = 0;
}

BezierPathLibart::BezierPathLibart(ArtBpath *other) : BezierPath()
{
	path = art_new(ArtBpath, 1);
	int i = 0;
	for(;other[i].code != ART_END; i++)
	{
		path[i] = other[i];
		art_renew(path, ArtBpath, i + 2);
	}
	path[i].code = ART_END;
}

BezierPathLibart::~BezierPathLibart()
{
	if(path)
		art_free(path);
}

float BezierPathLibart::getTotalLength() const
{
	float result = 0.0;
	return result;
}

Point BezierPathLibart::getPointAtLength(float /*distance*/) const
{
	Point ret;
	return ret;
}

double BezierPathLibart::chordLength(const BezierPath *prev) const
{
	const BezierPathLibart *lpath = static_cast<const BezierPathLibart *>(prev);
	ArtBpath *ppath = (lpath ? lpath->path : 0);
	if(!ppath)
		return 0.0;

	return sqrt((path->x3 - ppath->x3) *
				(path->x3 - ppath->x3) +
				(path->y3 - ppath->y3) *
				(path->y3 - ppath->y3));
}

double BezierPathLibart::polyLength(const BezierPath *prev) const
{
	const BezierPathLibart *lpath = static_cast<const BezierPathLibart *>(prev);
	ArtBpath *ppath = (lpath ? lpath->path : 0);
	if(!ppath)
		return 0.0;

	return sqrt((path->x1 - ppath->x3) *
				(path->x1 - ppath->x3) +
				(path->x1 - ppath->x3) *
				(path->x1 - ppath->x3)) +
		   sqrt((path->x2 - path->x1) *
				(path->x2 - path->x1) +
				(path->x2 - path->x1) *
				(path->x2 - path->x1)) +
		   sqrt((path->x3 - path->x2) *
				(path->x3 - path->x2) +
				(path->x3 - path->x2) *
				(path->x3 - path->x2));
}

double BezierPathLibart::length(double t) const
{
	// We cheat a bit...
	ArtVpath *vpath = art_bez_path_to_vec(path, 0.25);
	double total = 0.0;
	double x = 0.0, y = 0.0;
	for(int i = 0; vpath[i].code != ART_END; i++)
	{
		if(vpath[i].code == ART_MOVETO)
		{
			x = vpath[i].x;
			y = vpath[i].y;
		}
		else if(vpath[i].code == ART_LINETO)
		{
			double dx = x, dy = y;
			x = vpath[i].x;
			y = vpath[i].y;
			dx = x - dx;
			dy = y - dy;
			total += sqrt(pow(dx, 2) + pow(dy, 2));
		}
	}
	art_free(vpath);
	return total;
}

void BezierPathLibart::pointDerivativesAt(double t, Point *p, Point *d1, Point *d2) const
{
}

void BezierPathLibart::pointTangentNormalAt(double t, Point *p, Point *tn, Point *n) const
{
	double totallen = length() * t;
	// We cheat a bit...
	ArtVpath *vpath = art_bez_path_to_vec(path, 0.25);
	double total = 0.0;
	double x = 0.0, y = 0.0;
	for(int i = 0; vpath[i].code != ART_END; i++)
	{
		if(vpath[i].code == ART_MOVETO)
		{
			x = vpath[i].x;
			y = vpath[i].y;
		}
		else if(vpath[i].code == ART_LINETO)
		{
			double dx = x, dy = y;
			x = vpath[i].x;
			y = vpath[i].y;
			dx = x - dx;
			dy = y - dy;
			double seg_len = sqrt(pow(dx, 2) + pow(dy, 2));
			total += seg_len;
			if(total >= totallen)
			{
				double fract = 1 - (totallen - (total - seg_len)) / seg_len;
				if(p)
				{
					p->setX(x - dx * fract);
					p->setY(y - dy * fract);
				}
				// Calculate tangent
				if(tn)
				{
					tn->setX(dx);
					tn->setY(dy);
				}
				// Calculate normal vector.
				if(n)
				{
					// Calculate vector product of "binormal" x tangent
					// (0,0,1) x (dx,dy,0), which is simply (dy,-dx,0).
					n->setX(dy);
					n->setY(-dx);
				}
				return;
			}
		}
	}
	art_free(vpath);
}	   

// vim:ts=4:noet
